# coding: utf-8

# Copyright (c) Vidar Tonaas Fauske.
# Distributed under the terms of the Modified BSD License.

import pytest

import datetime
import pytz

from traitlets import TraitError

from ..trait_types import (
    time_to_json,
    time_from_json,
    datetime_to_json,
    datetime_from_json,
)


def test_time_serialize_none():
    assert time_to_json(None, None) == None


def test_time_serialize_value():
    t = datetime.time(13, 37, 42, 7000)
    assert time_to_json(t, None) == dict(
        hours=13, minutes=37, seconds=42, milliseconds=7
    )


def test_time_deserialize_none():
    assert time_from_json(None, None) == None


def test_time_deserialize_value():
    v = dict(hours=13, minutes=37, seconds=42, milliseconds=7)
    assert time_from_json(v, None) == datetime.time(13, 37, 42, 7000)


def test_datetime_serialize_none():
    assert datetime_to_json(None, None) == None


def test_datetime_serialize_value():
    t = datetime.datetime(2002, 2, 20, 13, 37, 42, 7000, pytz.utc)
    assert datetime_to_json(t, None) == dict(
        year=2002,
        month=1,  # Months are 0-based indices in JS
        date=20,
        hours=13,
        minutes=37,
        seconds=42,
        milliseconds=7,
    )


def test_datetime_serialize_non_utz():
    # Non-existent timezone, so it will never be the local one:
    tz = pytz.FixedOffset(42)
    t = datetime.datetime(2002, 2, 20, 13, 37, 42, 7000, tz)
    assert datetime_to_json(t, None) == dict(
        year=2002,
        month=1,  # Months are 0-based indices in JS
        date=20,
        hours=12,
        minutes=55,
        seconds=42,
        milliseconds=7,
    )


def test_datetime_deserialize_none():
    assert datetime_from_json(None, None) == None


def test_datetime_deserialize_value():
    tz = pytz.FixedOffset(42)
    v = dict(
        year=2002,
        month=1,  # Months are 0-based indices in JS
        date=20,
        hours=13,
        minutes=37,
        seconds=42,
        milliseconds=7,
    )
    assert datetime_from_json(v, None) == datetime.datetime(
        2002, 2, 20, 14, 19, 42, 7000, tz
    )



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

"""Contains the ValueWidget class"""

from .widget import Widget
from traitlets import Any


class ValueWidget(Widget):
    """Widget that can be used for the input of an interactive function"""

    value = Any(help="The value of the widget.")

    def get_interact_value(self):
        """Return the value for this widget which should be passed to
        interactive functions. Custom widgets can change this method
        to process the raw value ``self.value``.
        """
        return self.value

    def _repr_keys(self):
        # Ensure value key comes first, and is always present
        yield 'value'
        for key in super()._repr_keys():
            if key != 'value':
                yield key



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

from unittest import TestCase

from traitlets import TraitError

import ipywidgets as widgets


class TestBox(TestCase):

    def test_construction(self):
        box = widgets.Box()
        assert box.get_state()['children'] == []

    def test_construction_with_children(self):
        html = widgets.HTML('some html')
        slider = widgets.IntSlider()
        box = widgets.Box([html, slider])
        children_state = box.get_state()['children']
        assert children_state == [
            widgets.widget._widget_to_json(html, None),
            widgets.widget._widget_to_json(slider, None),
        ]

    def test_construction_style(self):
        box = widgets.Box(box_style='warning')
        assert box.get_state()['box_style'] == 'warning'

    def test_construction_invalid_style(self):
        with self.assertRaises(TraitError):
            widgets.Box(box_style='invalid')



# compatibility shim for ipykernel < 6.18
import sys
from IPython import get_ipython
import comm


def requires_ipykernel_shim():
    if "ipykernel" in sys.modules:
        import ipykernel

        version = ipykernel.version_info
        return version < (6, 18)
    else:
        return False


def get_comm_manager():
    if requires_ipykernel_shim():
        ip = get_ipython()

        if ip is not None and getattr(ip, "kernel", None) is not None:
            return get_ipython().kernel.comm_manager
    else:
        return comm.get_comm_manager()


def create_comm(*args, **kwargs):
    if requires_ipykernel_shim():
        from ipykernel.comm import Comm

        return Comm(*args, **kwargs)
    else:
        return comm.create_comm(*args, **kwargs)



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import inspect
import pytest

from ..widget_string import Combobox, Text

def test_combobox_creation_blank():
    w = Combobox()
    assert w.value == ''
    assert w.options == ()
    assert w.ensure_option == False


def test_combobox_creation_kwargs():
    w = Combobox(
        value='Chocolate',
        options=[
            "Chocolate",
            "Coconut",
            "Mint",
            "Strawberry",
            "Vanilla",
        ],
        ensure_option=True
    )
    assert w.value == 'Chocolate'
    assert w.options == (
            "Chocolate",
            "Coconut",
            "Mint",
            "Strawberry",
            "Vanilla",
        )
    assert w.ensure_option == True

def test_tooltip_deprecation():
    caller_path = inspect.stack(context=0)[1].filename
    with pytest.deprecated_call() as record:
        w = Text(description_tooltip="testing")
    assert len(record) == 1
    assert record[0].filename == caller_path

    with pytest.deprecated_call() as record:
        w.description_tooltip
    assert len(record) == 1
    assert record[0].filename == caller_path

    with pytest.deprecated_call() as record:
        w.description_tooltip == "testing"
    assert len(record) == 1
    assert record[0].filename == caller_path

    with pytest.deprecated_call() as record:
        w.description_tooltip = "second value"
    assert len(record) == 1
    assert record[0].filename == caller_path
    assert w.tooltip == "second value"

def test_on_submit_deprecation():
    with pytest.deprecated_call() as record:
        Text().on_submit(lambda *args: ...)
    assert len(record) == 1
    assert record[0].filename == inspect.stack(context=0)[1].filename
